@@ -1,9 +1,21 @@
 '''Provides an object model for a Singer Catalog.'''
-
 import json
 import sys
 
-from singer.schema import Schema
+from . import metadata as metadata_module
+from .bookmarks import get_currently_syncing
+from .logger import get_logger
+from .schema import Schema
+
+LOGGER = get_logger()
+
+
+def write_catalog(catalog):
+    # If the catalog has no streams, log a warning
+    if not catalog.streams:
+        LOGGER.warning("Catalog being written with no streams.")
+
+    json.dump(catalog.to_dict(), sys.stdout, indent=2)
 
 # pylint: disable=too-many-instance-attributes
 class CatalogEntry():
@@ -33,7 +45,9 @@ class CatalogEntry():
         return self.__dict__ == other.__dict__
 
     def is_selected(self):
-        return self.schema.selected  # pylint: disable=no-member
+        mdata = metadata_module.to_map(self.metadata)
+        # pylint: disable=no-member
+        return self.schema.selected or metadata_module.get(mdata, (), 'selected')
 
     def to_dict(self):
         result = {}
@@ -78,7 +92,7 @@ class Catalog():
 
     @classmethod
     def load(cls, filename):
-        with open(filename) as fp:  # pylint: disable=invalid-name
+        with open(filename, encoding="utf-8") as fp:
             return Catalog.from_dict(json.load(fp))
 
     @classmethod
@@ -116,3 +130,27 @@ class Catalog():
             if stream.tap_stream_id == tap_stream_id:
                 return stream
         return None
+
+    def _shuffle_streams(self, state):
+        currently_syncing = get_currently_syncing(state)
+
+        if currently_syncing is None:
+            return self.streams
+
+        matching_index = 0
+        for i, catalog_entry in enumerate(self.streams):
+            if catalog_entry.tap_stream_id == currently_syncing:
+                matching_index = i
+                break
+        top_half = self.streams[matching_index:]
+        bottom_half = self.streams[:matching_index]
+        return top_half + bottom_half
+
+
+    def get_selected_streams(self, state):
+        for stream in self._shuffle_streams(state):
+            if not stream.is_selected():
+                LOGGER.info('Skipping stream: %s', stream.tap_stream_id)
+                continue
+
+            yield stream
